from typing import Any, TypeVar, Union, get_args, get_origin

TypeHint = TypeVar("TypeHint")


def is_type_required(type_hint: type[Any]) -> bool:
    """
    Returns bool value, whether argument's type hint is required:
    str, int, Union[str, int]...
    Or not required:
    Optional[str], str | None, Union[str, None]...
    """
    return get_origin(type_hint) is not Union and type(None) not in get_args(type_hint)


def get_type_from_optional(type_hint: type[Any]) -> type[Any]:
    return get_args(type_hint)[0]



# import json
# from enum import Enum
# from typing import Any, Union

# from openapi_schema_pydantic.v3.v3_0_3 import OpenAPI
# from openapi_schema_pydantic.v3.v3_0_3.parameter import Parameter
# from openapi_schema_pydantic.v3.v3_0_3.util import (
#     PydanticSchema,
#     Schema,
#     construct_open_api_with_schema_class,
# )

# from aws_spy.core.schemas import Methods, SpyRoute
# from aws_spy.core.types import is_type_required

# SCHEMA_PARAM = {
#     int: "integer",
#     str: "string",
#     bool: "boolean",
#     float: "number",
#     Enum: "string",
# }


# def _get_path_item(method: Methods, route: SpyRoute) -> dict[str, Any]:
#     item = {
#         "responses": {
#             route.status_code: {
#                 "description": "Successful response",
#                 "content": {
#                     "application/json": {
#                         "schema": PydanticSchema(schema_class=route.response_class) if route.response_class else {}
#                     }
#                 },
#             },
#             422: {
#                 "description": "Validation Error",
#                 "content": {"application/json": {"schema": {"$ref": "#/components/schemas/SPY422ErrorResponse"}}},
#             },
#         }
#     }
#     item["parameters"] = [  # type: ignore
#         Parameter(
#             name=route_param.name,
#             required=is_type_required(route_param.annotation),
#             param_in=route_param.in_.in_,
#             param_schema=Schema(
#                 type=SCHEMA_PARAM.get(route_param.annotation, "string"),
#                 enum=route_param.enum,
#             ),
#         )
#         for route_param in route.header_params + route.path_params + route.query_params
#     ]
#     if route.tags:
#         item["tags"] = route.tags  # type: ignore
#     if route.summary:
#         item["summary"] = route.summary  # type: ignore
#     if route.description:
#         item["description"] = route.description  # type: ignore
#     if method not in (Methods.GET, Methods.DELETE):
#         item.update(
#             {
#                 "requestBody": {
#                     "content": {  # type: ignore
#                         "application/json": {
#                             "schema": PydanticSchema(schema_class=route.request_body) if route.request_body else {}
#                         }
#                     }
#                 }
#             }
#         )

#     return item


# def construct_base_open_api(title: str, version: str | None, routes: dict[str, dict[Methods, SpyRoute]]) -> OpenAPI:
#     info = {"title": title}
#     if version is not None:
#         info["version"] = version
#     return OpenAPI.parse_obj(
#         {
#             "info": info,
#             "paths": {
#                 path: {method: _get_path_item(method, route) for method, route in route_dict.items()}
#                 for path, route_dict in routes.items()
#             },
#             "components": {
#                 "schemas": {
#                     "SPY422SingleError": {
#                         "title": "SPY 422 Single Error Schema",
#                         "required": ["message"],
#                         "type": "object",
#                         "properties": {
#                             "message": {
#                                 "title": "Error message",
#                                 "type": "string",
#                             }
#                         },
#                     },
#                     "SPY422ErrorResponse": {
#                         "title": "SPY 422 Error Response",
#                         "required": ["error"],
#                         "type": "object",
#                         "properties": {
#                             "errors": {
#                                 "type": "array",
#                                 "items": {"$ref": "#/components/schemas/SPY422SingleError"},
#                                 "example": [
#                                     "Wrong type received at: age. Expected: integer",
#                                     "Value not found at: name",
#                                     "Required parameter authorization not found in header.",
#                                     "user_id should be UUID type.",
#                                 ],
#                             }
#                         },
#                     },
#                 }
#             },
#         }
#     )


# def get_openapi(title: str, version: str | None, routes: dict[str, dict[Methods, SpyRoute]]) -> dict[str, Any]:
#     return json.loads(
#         construct_open_api_with_schema_class(construct_base_open_api(title, version, routes)).json(
#             by_alias=True, exclude_none=True, indent=2
#         )
#     )



from aws_spy.core import params


def Query(name: str | None = None):  # noqa: N802
    return params.QueryClass(name)


def Path(name: str | None = None):  # noqa: N802
    return params.PathClass(name)


def Header(name: str | None = None):  # noqa: N802
    return params.HeaderClass(name)



import json
from typing import Any, Union

import pytest
from pydantic import BaseModel

from aws_spy.core.schemas import SpyRoute
from aws_spy.responses import ErrorResponse, JSONResponse, RAWResponse


class ExampleResponseClass(BaseModel):
    x: int
    y: int


class SameAsExampleResponseClass(BaseModel):
    x: int
    y: int


@pytest.mark.parametrize(
    ["data", "response_class", "expected_response_body", "additional_headers"],
    [
        (
            ExampleResponseClass(x=10, y=15),
            ExampleResponseClass,
            json.dumps({"x": 10, "y": 15}),
            None,
        ),
        (
            SameAsExampleResponseClass(x=10, y=15),
            ExampleResponseClass,
            json.dumps({"x": 10, "y": 15}),
            None,
        ),
        (
            {"x": 10, "y": 15},
            ExampleResponseClass,
            json.dumps({"x": 10, "y": 15}),
            None,
        ),
    ],
)
def test_json_response(
    data: Union[BaseModel, dict[str, Any]],
    response_class: type[BaseModel],
    expected_response_body: str,
    additional_headers: Union[dict[str, str], None],
) -> None:
    expected_headers = {
        "Content-Type": "application/json",
    }
    if additional_headers:
        expected_headers.update(additional_headers)

    route = SpyRoute(
        name="random",
        path="/path",
        method="get",
        handler=lambda: "nothing",
        status_code=202,
        response_class=response_class,
    )
    response_cls = JSONResponse(data, additional_headers)
    response_cls.route = route
    response = response_cls.response
    assert response["statusCode"] == 202
    assert response["body"] == expected_response_body
    assert response["headers"] == expected_headers



import json
from enum import Enum
from typing import Any, Literal, Union

import pytest
from pydantic import BaseModel, Field

from aws_spy.core.event_utils import export_params_from_event, export_request_body
from aws_spy.core.params_alias import Header
from aws_spy.core.schemas import ParamSchema


class ExampleEnum(str, Enum):
    example = "example"


class ExampleRequestBody(BaseModel):
    a: int
    b: str
    c: bool
    d: Union[int, None]
    example: Union[ExampleEnum, None] = Field(None)


def gpm(
    name: str,
    arg_name: str,
    annotation: type,
    is_required: bool = True,
    enum: Union[list[str], None] = None,
) -> ParamSchema:
    return ParamSchema(
        name=name,
        arg_name=arg_name,
        in_=Header(),
        annotation=annotation,
        is_required=is_required,
        enum=enum,
    )


@pytest.mark.parametrize(
    [
        "in_event_params",
        "expected_params",
        "type_",
        "expected_params_result",
        "expected_errors_result",
    ],
    [
        (None, [], "path", {}, []),
        [
            {"user_id": "valid user id"},
            [gpm("user_id", "user_id", str)],
            "path",
            {"user_id": "valid user id"},
            [],
        ],
        [
            {"user_id": "valid user id"},
            [gpm("user_id", "user_id", int)],
            "path",
            {},
            ["user_id should be int type."],
        ],
        [
            None,
            [gpm("user_id", "user_id", int)],
            "path",
            {},
            ["Required parameter user_id not found in path."],
        ],
        [
            None,
            [gpm("user_id", "user_id", int, False)],
            "path",
            {"user_id": None},
            [],
        ],
        [
            {"user_id": "example"},
            [gpm("user_id", "user_id", ExampleEnum)],
            "path",
            {"user_id": "example"},
            [],
        ],
        [
            {"user_id": "valid user id"},
            [gpm("user_id", "user_id", ExampleEnum)],
            "path",
            {},
            ["user_id should be ExampleEnum type."],
        ],
    ],
)
def test_export_params_from_event_success(
    in_event_params: Union[dict[str, Any], None],
    expected_params: list[ParamSchema],
    type_: Literal["path", "header", "query"],
    expected_params_result: dict[str, Any],
    expected_errors_result: list[str],
) -> None:
    params, errors = export_params_from_event(in_event_params, expected_params, type_)
    assert params == expected_params_result
    assert errors == expected_errors_result


@pytest.mark.parametrize(
    ["body", "expected_errors"],
    [
        (json.dumps({"a": 1, "b": "string", "c": True, "d": None}), []),
        ["", ["Request body is empty!"]],
        (
            json.dumps({"a": "str", "b": "string", "c": "str", "d": None}),
            [
                "Wrong type received at: a. Expected: integer",
                "Wrong type received at: c. Expected: bool",
            ],
        ),
        (
            json.dumps(
                {"a": 1, "b": "string", "c": True, "d": "10", "example": "not example"}
            ),
            ["Wrong type received at: example. Expected: enum"],
        ),
        (
            json.dumps({"b": "string", "c": True, "d": "10", "example": "example"}),
            ["Value not found at: a"],
        ),
    ],
)
def test_export_request_body(body: str, expected_errors: list[str]) -> None:
    request_body, errors = export_request_body(body, ExampleRequestBody)
    assert errors == expected_errors
    if not errors:
        assert isinstance(request_body, ExampleRequestBody)
